package com.unis.module.coding.controller.admin.dwxx.vo;

import com.alibaba.excel.annotation.ExcelIgnoreUnannotated;
import com.alibaba.excel.annotation.ExcelProperty;
import com.unis.module.coding.controller.admin.cwxx.vo.CwxxRespVO;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

@Schema(description = "管理后台 - 单位信息 Response VO")
@Data
@ExcelIgnoreUnannotated
public class DwAndKqRespVO {

    @Schema(description = "主键", requiredMode = Schema.RequiredMode.REQUIRED, example = "30392")
    private Long id;

    @Schema(description = "单位名称")
    private String dwmc;

    @Schema(description = "统一信用代码")
    private String tyxydm;

    @Schema(description = "法定代表人")
    private String fddbr;

    @Schema(description = "身份证号")
    private String sfzh;

    @Schema(description = "联系方式")
    private String lxfs;

    @Schema(description = "注册日期")
    private LocalDateTime zcrq;

    @Schema(description = "邮政编码")
    private String yzbm;

    @Schema(description = "注册地址")
    private String zcdz;

    @Schema(description = "企业性质")
    private String qyxz;

    @Schema(description = "上传营业执照")
    private String license;

    @Schema(description = "单位下库区信息列表")
    private List<Kq> kqList;

    @Schema(description = "库区下仓房信息列表")
    private List<Cf> cfList;

    @Schema(description = "财务信息列表")
    private List<Cw> cwList;

    // 内部子类，包含单位下库区信息
    @Data
    @Schema(description = "单位下库区信息")
    public static class Kq {
        @Schema(description = "库区ID")
        private Long kqId;

        @Schema(description = "库区名称")
        private String kqmc;

        @Schema(description = "库区编码")
        private String kqdm;

        @Schema(description = "邮政编码")
        private String yzbm;

        @Schema(description = "省")
        private String sheng;

        @Schema(description = "市")
        private String shi;

        @Schema(description = "县")
        private String xian;

        @Schema(description = "省名称")
        private String shengName;

        @Schema(description = "市名称")
        private String shiName;

        @Schema(description = "县名称")
        private String xianName;

        @Schema(description = "具体地址")
        private String jtdz;

        @Schema(description = "库区负责人")
        private String kqfzr;

        @Schema(description = "联系电话")
        private String lxdh;

        @Schema(description = "占地面积")
        private BigDecimal zdmj;

        @Schema(description = "库区状态")
        private String kqzt;

        @Schema(description = "租仓参考价(元)")
        private Integer zcckj;

        @Schema(description = "委托保管参考价(元/吨)")
        private Integer wtbgckj;

        @Schema(description = "经度")
        private BigDecimal jd;

        @Schema(description = "纬度")
        private BigDecimal wd;

        @Schema(description = "上传库区图id")
        private String resid;
    }

    // 内部子类，包含库区下仓房信息
    @Data
    @Schema(description = "库区下仓房信息")
    public static class Cf {
        @Schema(description = "仓房id")
        private Long cfId;

        @Schema(description = "库区名称")
        private String kqmc;

        @Schema(description = "仓房名称")
        private String cfmc;

        @Schema(description = "仓房类型")
        private String cflx;

        @Schema(description = "仓房编号")
        private String cfbm;

        @Schema(description = "仓房状态")
        private String cazt;

        @Schema(description = "设计仓容")
        private BigDecimal sjcr;

        @Schema(description = "其他仓型说明")
        private String qtcxsm;

        @Schema(description = "客户名称(单位名称)")
        private String khmc;

        @Schema(description = "客户联系电话")
        private String khlxdh;

        @Schema(description = "客户身份证号(社会统一信用代码)")
        private String khbh;
    }

    // 内部子类，包含库区下财务信息
    @Data
    @Schema(description = "库区下财务信息")
    public static class Cw {
        @Schema(description = "财务ID")
        private Long cwId;

        @Schema(description = "库区名称")
        private String kqmc;

        @Schema(description = "是否国有及国有控股(0:是,1不是)")
        private String sfgykg;

        @Schema(description = "上一年度总资产(万元)")
        private String syndzzc;

        @Schema(description = "财年信息")
        private List<Cwy> cwyList;

        // 内部子类，包含财务下财年信息
        @Data
        @Schema(description = "财务下财年信息")
        public static class Cwy {
            @Schema(description = "财年ID")
            private Long cwyId;

            @Schema(description = "财年")
            private Integer cn;

            @Schema(description = "总负债(万元)")
            private BigDecimal zfz;

            @Schema(description = "营业收入(万元)")
            private BigDecimal yysr;

            @Schema(description = "净利润(万元)")
            private BigDecimal jlr;
        }
    }
}